
/*
 * New Mail Indicator extension for Gnome 3.x
 * Copyright 2018 fthx
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Main = imports.ui.main;
const Lang = imports.lang;
const MessageTray = imports.ui.messageTray;
const Util = imports.misc.util;
const GLib = imports.gi.GLib;

// get the default mail client's app.desktop filename (and remove the blank space at last position...)
let defaultMailAppFilename = GLib.spawn_command_line_sync("xdg-mime query default x-scheme-handler/mailto")[1].toString().slice(0,-1);
// get the default mail client's executable command
let defaultMailAppExe;
switch (defaultMailAppFilename) {
  case "thunderbird.desktop":
  case "mozilla-thunderbird.desktop":
    defaultMailAppExe = "thunderbird";
    defaultMailAppName = "Thunderbird";
    break;
  case "org.gnome.Evolution.desktop":
    defaultMailAppExe = "evolution -c mail";
    defaultMailAppName = "Evolution";
    break;
  case "org.gnome.Geary.desktop":
    defaultMailAppExe = "geary";
    defaultMailAppName = "Geary";
    break;
  case "mailspring_mailspring.desktop":
  	defaultMailAppExe = "mailspring";
  	defaultMailAppName = "Mailspring";
  	break;
  default:
    // if nothing is found
    Main.notify("New Mail Indicator: error, no known email client found.");
};
// for debug purposes you can uncomment the next line: it will pop-up the default mail client above names
//Main.notify(defaultMailAppFilename+"  "+defaultMailAppExe+"  "+defaultMailAppName);

const MessageCounterIndicator = new Lang.Class({

    Name: 'MailIndicator',

    _init: function() {
    	
        this.actor = new St.Bin({style_class: 'panel-button', visible: true, 
        	reactive: true, can_focus: true, track_hover: true});
        
        this.icon = new St.Icon({icon_name: 'mail-unread-symbolic', style_class: 'system-status-icon'});
        							
        this.actor.set_child(this.icon);
        
        this._sources = [];

        Main.messageTray.connect('source-added', Lang.bind(this, this._onSourceAdded));
        Main.messageTray.connect('source-removed', Lang.bind(this, this._onSourceRemoved));
        Main.messageTray.connect('queue-changed', Lang.bind(this, this._updateCount));

        let sources = Main.messageTray.getSources();
        sources.forEach(Lang.bind(this, function(source) {this._onSourceAdded(null, source);}));
        
        this.actor.connect('button-press-event', this._startDefaultMailApp);
        
    },

    _onSourceAdded: function(tray, source) {
        source.connect('count-updated', Lang.bind(this, this._updateCount));
        this._sources.push(source);
        this._updateCount();
    },

    _onSourceRemoved: function(tray, source) {
        this._sources.splice(this._sources.indexOf(source), 1);
        this._updateCount();
    },
    
    _startDefaultMailApp: function() {
    	Util.spawnCommandLine(defaultMailAppExe);
	},

    _updateCount: function() {
        let count = 0;

        this._sources.forEach(Lang.bind(this,
            function(source) {
            	if (source.title.includes(defaultMailAppName)) {
                	count++;
                }
            }
    	));
            
        let mail_icon;
        
        if (count > 0) {
        	mail_icon = 'mail-mark-important-symbolic';
        } else {
        	mail_icon = 'mail-unread-symbolic';
        }; 
        this.icon.icon_name = mail_icon;
        
    }
});

let count_indicator;

function init() {
}

function enable() {

	let centerBox = Main.panel._centerBox;
    count_indicator = new MessageCounterIndicator();
    centerBox.add_actor(count_indicator.actor);

}

function disable() {
    
	let centerBox = Main.panel._centerBox;
	centerBox.remove_actor(count_indicator.actor);
    
}
